#!/usr/bin/env python

global errorlevel
errorlevel = 0

import config, sys, os
sys.path += [os.getcwd()]

ingui = config.enable_dbus

# The module loader
from common import my_import

lp = my_import("lang." + config.lang_pack + '.text').main()
startwd = os.getcwd()
if config.auto_cls:
    os.system("clear")
    # TODO: Make this function wipe the screen with curses


try: 
    import subprocess, readline, time, datetime, string, threading, random, common
    from errordb import errnos
    # import the variable system for %variables%
    import varies
except ImportError, e: 
    print lp.get('importerror') + str(e)
    exit(2)


# add this folder to sys.path so stuff (config and plugins) can be accessed even when cded somewhere else

# import saved history
try: readline.read_history_file(os.getenv("HOME") + "/.dosprompt_history")
except: pass # just ignore it if it doesn't exist yet

def generate_random_seq(): # generate a random amount of random characters for the %random% variable
    length = random.randint(1, 10)
    out = ''
    for x in range(0, length + 1):
        which = random.randint(1, 2)
        if which == 1: # string
            out += string.letters[random.randint(0, 51)]
        else: # int
            out += str(random.randint(0, 9))
    return out


# Here, we will define a function for running the faux windows commands
# The commands and their functions are stored in the plugins folder.
# As the plugin file is run as a command (not imported), you can use any language you'd like (as long as it has a shebang [#!]) (or it can be a binary)
    # EDIT: python files are automatically detected by the .py extension and imported now, so you can have more control over DOSprompt with a very simple python API
# Anything printed by the plugin program will be printed when the user calls its alias from DOSprompt, and any arguments will be passed to it.
commands = os.listdir("plugins/")

def run(line):
    global errorlevel
    name = line[0].replace('.exe', '')
    if name + '.py' in commands:
        
        plugin = my_import('plugins.' + name)
        if name not in config.no_reload: reload(plugin)
        if 'main' in dir(plugin):
            if not name in config.need_errorlevel: out = plugin.main(line, startwd, config) # in a plugin, you must accept all of these arguments in your "main" function
            else: out = plugin.main(line, startwd, config, errorlevel)
            if type(out) == str: 
                if out.startswith('!goto: '): return out.split('!goto: ')[1]
                print out
            elif type(out) == type(my_import): 
                try: out() 
                except: print 'Unexpected fatal error in plugins main function'
            elif type(out) == int: errorlevel = out
            elif type(out) == tuple and len(out) == 2: # it can also return a two-tuple of (string, errno)
                print out[0]
                errorlevel = out[1]
            elif type(out) == type(None): pass
            else: print 'DOSprompt plugin "%s" returned an invalid type: "%s". Please report this at https://launchpad.net/dosprompt/' % (name, str(type(out)))
    elif name.startswith('cd') and line[0].startswith("cd.") or line[0].startswith("cd\\"):
        name = 'cd'
        plugin = my_import('plugins.' + name)
        if name not in config.no_reload: reload(plugin)
        if 'main' in dir(plugin):
            out = plugin.main(line, startwd, config) # in a plugin, you must accept all of these arguments in your "main" function
            if type(out) == str: print out
            elif type(out) == type(my_import): out() # a plugin's "main" function should not return anything other than a function or a str
            #                 ^^ this thing is just here because 'function' is not defined, so this is the only way i know of. If you have a better way, please fix this.
    else:
        errorlevel = 10000
        try: errorlevel = subprocess.call([startwd + "/plugins/" + line[0].replace('.exe', '')] + line[1:])
        except OSError, e: 
            if e.errno in errnos.keys():
                print "Unix error " + str(e.errno) + ':', errnos.get(e.errno, None)
            else: print lp.get('not_found') # TODO: change this to a more DOSey message
            errorlevel = e.errno
        except KeyboardInterrupt: pass
        if running_batch and errorlevel != 0: 
            print 'Error, Exiting.'
            sys.exit(1)

        
for plugin in config.auto_start:
    run(plugin.split())

if 'setcwd' in sys.argv:
    n = sys.argv.index('setcwd')
    try: os.chdir(sys.argv[n + 1])
    except IndexError: print "setcwd must have an path"

#import fcntl
# make stdin a non-blocking file
#fd = sys.stdin.fileno()
#fl = fcntl.fcntl(fd, fcntl.F_GETFL)
#fcntl.fcntl(fd, fcntl.F_SETFL, fl | os.O_NONBLOCK)

# user input handling thread
#while 1:
#      try: input = sys.stdin.readline()
#      except: continue
#      handleInput(input)
      
      
# before entering the main loop, do a quick check to see if there is a batch script to run
if len(sys.argv) > 1: path = sys.argv[1]
else: path = 'nonexistant.thing.to.do.a.buggy.thing.that.should.not.be.done.but.this.file.still.better.not.exist!'
if path.endswith(".bat") and os.path.exists(path) and config.developer_mode:
    print 'Remember, the batch script runner is still in alpha stage, it probably won\'t work!'
    running_batch = True
    batchf = open(path)
    batch = batchf.readlines()
    batchf.close()
    marks = {} # place to stick :marks for the special goto command
    x = 0
    for line in batch:
        if line.startswith(':'):
            marks.update({line.split(':')[1].strip(): x})
            batch[x] = 'rem'
        x += 1
            
else: 
    running_batch = False
    batch = []

    
if config.color and not running_batch: print "\033[3" + str(config.fg_color) + 'm' + "\033[4" + str(config.bg_color) + 'm' + u"""DOSprompt [Version %s]
Copyright \xa9(C) 2011 M. Smith, GPLv3.  Some rights reserved.""" % common.version

elif not config.color and not running_batch: print u"""DOSprompt [Version %s]
Copyright \xa9(C) 2011 M. Smith, GPLv3.  Some rights reserved.""" % common.version


# Loop variables #
count = 0
echo = True
goto = None

##################
# load real system variables into the awesome %variable% handler
for varname in os.environ.keys():
        varies.var(varname, os.environ[varname])
        
# now we enter the infinite loop. To increase responsiveness, make sure anything that probably won't change during one session is up there, not down here.
while 1: # 1 always is... existant ;)
    if running_batch and count == len(batch): break
    
    if config.reload_conf:
        reload(config)
    cwd = os.getcwd() # the Current Working Directory

    if cwd.startswith('/media/') or cwd.startswith('/mnt/'): dospath = common.dos_path(cwd, True)
    else: dospath = 'C:' + common.dos_path(cwd).replace('/', '\\')
    drive = dospath.split(':')[0]

    rendered_prompt = config.prompt.replace("$n", drive).replace("$p", dospath).replace("$user", os.getenv("LOGNAME")).replace("$hostname", str(os.popen("hostname").read().replace("\n", ""))).replace("$$", "$").replace("$d", str(datetime.datetime.now().date())).replace("$b", "|").replace("$t", str(datetime.datetime.now().time()))
    
    if config.color: dosprompt = "\033[3" + str(config.fg_color) + 'm' + "\033[4" + str(config.bg_color) + 'm' + rendered_prompt
    else: dosprompt = rendered_prompt
    
    ################################
    #   start the dbus main loop   #
  #  if config.developer_mode: dbusloop.start()
    #                              #
    ################################
    
    if not running_batch:
        try: cmd = raw_input(dosprompt)
        except EOFError:
            print "\n"
            break
        except KeyboardInterrupt:
            cmd = "echo"
            pass
    # this handles the running of batch scripts
    else:
        cmd = batch[count].strip()
        if cmd == '@echo off': 
            echo = False
            cmd = 'rem'
        elif cmd == '@echo on': 
            echo = True
            cmd = 'rem'
        #if goto and goto in marks.keys():
        #    count = marks[goto]
        #    cmd = 'rem'
        #    goto = None

        if echo and not cmd.startswith("#") and not cmd.startswith("rem"): print cmd
        if cmd.startswith("#"): cmd = "rem"

    if config.reload_env: reload(os)
    for varname in os.environ.keys():
        varies.get(varname).update(os.environ[varname])
    # force-reset pwd and cd and dosify them
    varies.get('pwd').update(common.dos_path(os.getcwd()).replace('/', '\\'))
    varies.get('cd').update(common.dos_path(os.getcwd()).replace('/', '\\'))
    # and update the dbus varies
    if ingui: threading.Thread(target = varies.dbus_update).start()
    
    ################################
    #   stop the dbus main loop   #
  #  if config.developer_mode: dbusloop.stop()
    #                              #
    ################################
    if not cmd or cmd == " ":
        cmd = "rem"

    if cmd == "dosp plugindex":
        print lp.get('plugindexing')
        commands = os.listdir(startwd + "/plugins/")
        cmd = "rem"
        line = ['rem']
        
    elif cmd == "dosp lsplugins":
        commands.sort()
        for command in commands:
            if command.endswith(".py") and not command.startswith('_'):
                print '\033[32m' + command.replace(".py", '') + '\033[0m'
            elif not command.endswith(".pyc") and not command.startswith('_'): print command
        cmd = "rem"
        line = ['rem']
        
    elif cmd == 'exit' or cmd == 'quit':
        break
        cmd = "rem"
        line = ['rem']
    
    if '%' in cmd and len(cmd.split('%')) > 2:
        # magic variables
        magic_vars = ['random']
        if '%random%' in cmd:
            cmd = cmd.replace('%random%', generate_random_seq())
                    
        
        for pv in cmd.split('%'):
            if not pv.startswith(" ") and not pv.endswith(" ") and not len(pv.split()) == 0 and len(pv.split()) == 1 and not pv in magic_vars: 
                cmd = cmd.replace('%' + pv + '%', str(varies.get(pv, 'Not Defined')))

    line = cmd.split()
    # fix for LP bug #759301 # not especially functional
    #tline = "'".join(cmd.split('"')).split("'")
    #line = tline[0].strip().split()
    #if len(tline) > 1:
    #    for a in tline[1:-1]:
    #        line += ['"' + a + '"']
    if line[0] in config.aliases.keys() or line[0].replace('.exe', '') in config.aliases.keys():
        reel = config.aliases.get(line[0], 'error')
        try: rest = line[2:]
        except IndexError: rest = []
        if len(reel.split()) == 1: line[0] = reel
        elif len(reel.split()) > 1: line = reel.split() + rest

                
    if line[0] in commands or line[0] + '.py' in commands or line[0].replace(".exe", '') in commands or line[0].startswith("cd.") or line[0].startswith("cd\\") or line[0].replace('.exe', '.py') in commands:
        try: 
            goto = run(line)
            if running_batch and goto: 
                count = marks[goto]
        except:
            print 'The plugin %s had a fatal error! Please report this at https://launchpad.net/dosprompt/'
            if config.developer_mode: 
                print 'Developer mode enabled, run again to spew errors? (dosprompt will probably die)'
                yn = raw_input('[Y/n]')
                if yn.lower().strip().strip(string.punctuation).strip() in lp.get('yesses') or yn == '':
                    run(line)
                else: print 'Ok, please report it with as much information as you can'
        
    else:
        try: errorlevel = subprocess.call(line)
        except OSError, e: 
            print lp.get('not_found')
            errorlevel = e.errno
        except KeyboardInterrupt:
            pass
        #    print 'Error, Exiting.'
        #    sys.exit(1)
    varies.var('errorlevel', str(errorlevel))
    count += 1


    
readline.write_history_file(os.path.normpath(os.getenv("HOME") + "/.dosprompt_history"))

