#!/usr/bin/env python
from datetime import datetime
import socket, os, config, common, sys

_varlst = {}

class var(object):
    def __init__(self, varname, value, mod='', vtype=str):
        '''Create A Magic variable object for DOSprompt's %variable% system'''
        self.name = varname.lower()
        self.time_set = datetime.now()
        self.value = value
        self.mod = mod # the path to a module to update (like config.py) (not required)
        self.type = vtype
        # set it through dbus (TODO: Make this actually work!!!)
        #os.system("dbus-send --print-reply --dest=org.dosprompt.Varies /org/dosprompt/Varies/VarSync org.dosprompt.VariesInterface.setvar string:'%s' string:'%s' &" % (self.name, value))
        _varlst.update({self.name: self})
        
    def update(self, value):
        oldval = self.value
        self.value = value
        # update it in dbus  (TODO: Make this actually work!!!)
        #os.system("dbus-send --print-reply --dest=org.dosprompt.Varies /org/dosprompt/Varies/VarSync org.dosprompt.VariesInterface.setvar string:'%s' string:'%s' &" % (self.name, value))
        # send a varchanged signal through dbus
        eo.varchanged(self.name, value)
        _varlst.update({self.name: self})
        
        if self.mod and os.path.exists(self.mod): # is there a module to update too?
            cf = open(self.mod)
            cfdata = cf.read()
            cf.close()
            cfw = open(self.mod, 'w') # reopen the file in overwrite mode
            prefix = ''
            if self.type == str: prefix = '"'
            cfw.write(cfdata.replace('%s = %s%s' % (self.name, prefix, oldval), '%s = %s%s' % (self.name, prefix, self.value)))
            
        
    def __str__(self):
        return self.value
        
        
def get(name, err=None, otype=None): # otype is a type to try to convert the value to
    if name.lower() in _varlst.keys(): 
        if otype: return otype(_varlst[name.lower()].value)
        else: return _varlst[name.lower()]
    else: return err

def search(name, err=None):
    name = name.lower()
    for key in _varlst.keys(): 
        if name in key or name in _varlst[key].value.lower():
            print key + '=' + get(key).value
            
def dbus_update(): # load all the variables into dbus
    for name in _varlst.keys():
        # for now, only if the var doesn't have a mod to update
        if not _varlst[name].mod: os.popen("dbus-send --print-reply --dest=org.dosprompt.Varies /org/dosprompt/Varies/VarSync org.dosprompt.VariesInterface.setvar string:'%s' string:'%s'" % (name, _varlst[name]))
            

# Add everything from config as variables
for opt in dir(config):
    if not opt.startswith('_'):
        var(opt, str(getattr(config, opt)), mod=config.__file__.replace('.pyc', '.py'), vtype=type(getattr(config, opt)))
        

# Add many common windows variables
# TODO: Change the following variables to be useful for linux
var('ALLUSERSPROFILE', 'C:\ProgramData')
var('APPDATA', 'C:\Users\%s\AppData\Roaming' % os.getenv("USER"))
var('CommonProgramFiles', 'C:\Program Files\Common Files')
var('CommonProgramFiles(x86)', 'C:\Program Files (x86)\Common Files')
var('CommonProgramW6432', 'C:\Program Files\Common Files')
var('COMPUTERNAME', socket.gethostname())
var('HOMEDRIVE', '/')
var('cd', common.dos_path(os.getcwd()).replace('/', '\\'))

import dbus, dbus.service, gobject, threading
from dbus.mainloop.glib import DBusGMainLoop
from dbus.service import method, signal

# This and a few other snippets give a much richer plugin API with DBus
#######################################################################
# start dbus main loop
DBusGMainLoop(set_as_default=True)

# set some dbus options
DBUS_BUSNAME = "org.dosprompt.Varies"
DBUS_IFACE = 'org.dosprompt.VariesInterface'
DBUS_PATH = '/org/dosprompt/Varies/VarSync'

class ExampleObject(dbus.service.Object):

    @method(dbus_interface=DBUS_IFACE, in_signature="s")
    def getvar(self, req):
        print 'DBus request for', req, 'variable'
        return str(get(req))
        
    @method(dbus_interface=DBUS_IFACE, in_signature="ss")
    def setvar(self, vari, val):
        print 'DBus setting %s to %s' % (vari, val)
        if vari in _varlst.keys(): get(vari).update(val)
        else: var(vari, val)
        
    @signal(dbus_interface=DBUS_IFACE, signature="ss")
    def varchanged(self, vari, val):
        pass
        

session_bus = dbus.SessionBus()
busname = dbus.service.BusName(DBUS_BUSNAME, bus=session_bus)
eo = ExampleObject(object_path=DBUS_PATH, bus_name=busname)

loop = gobject.MainLoop()

#######################################################################

def start():
    loop.run()
    
def stop():
    loop.quit()


if __name__ == '__main__':
    if not config.developer_mode: exit()
    if len(sys.argv) == 3:
        if sys.argv[1] == 'set':
            var(sys.argv[2].split('=')[0], sys.argv[2].split('=')[1])
        if sys.argv[1] == 'get':
            print get(sys.argv[2])
    elif len(sys.argv) == 2:
        if sys.argv[1] == 'sync':
            try: start()
            except: stop()
   
    
    
    
